@@ -94,7 +94,26 @@ class ServiceDescriptor(object):
         )
 
     @staticmethod
-    def nft_sales_service_descriptor(attributes, service_endpoint):
+    def nft_sales_service_descriptor(attributes, service_endpoint, is_1155=True):
+        """
+        NFT Sales service descriptor.
+
+        :param attributes: attributes of the nft sales service, dict
+        :param service_endpoint: identifier of the service inside the asset DDO, str
+        :return: Service descriptor.
+        """
+        if is_1155:
+            return (
+                ServiceTypes.NFT_SALES,
+                {'attributes': attributes, 'serviceEndpoint': service_endpoint}
+            )
+        return (
+            ServiceTypes.NFT721_SALES,
+            {'attributes': attributes, 'serviceEndpoint': service_endpoint}
+        )
+    
+    @staticmethod
+    def nft_sales_with_access_service_descriptor(attributes, service_endpoint):
         """
         NFT Sales service descriptor.
 
@@ -103,12 +122,31 @@ class ServiceDescriptor(object):
         :return: Service descriptor.
         """
         return (
-            ServiceTypes.NFT_SALES,
+            ServiceTypes.NFT_SALES_WITH_ACCESS,
             {'attributes': attributes, 'serviceEndpoint': service_endpoint}
         )
 
     @staticmethod
-    def nft_access_service_descriptor(attributes, service_endpoint):
+    def nft_access_service_descriptor(attributes, service_endpoint, is_1155=True):
+        """
+        NFT Access service descriptor.
+
+        :param attributes: attributes of the nft access service, dict
+        :param service_endpoint: identifier of the service inside the asset DDO, str
+        :return: Service descriptor.
+        """
+        if is_1155:
+            return (
+                ServiceTypes.NFT_ACCESS,
+                {'attributes': attributes, 'serviceEndpoint': service_endpoint}
+            )
+        return (
+            ServiceTypes.NFT721_ACCESS,
+            {'attributes': attributes, 'serviceEndpoint': service_endpoint}
+        )
+
+    @staticmethod
+    def nft_access_proof_service_descriptor(attributes, service_endpoint):
         """
         NFT Access service descriptor.
 
@@ -117,7 +155,7 @@ class ServiceDescriptor(object):
         :return: Service descriptor.
         """
         return (
-            ServiceTypes.NFT_ACCESS,
+            ServiceTypes.NFT_ACCESS_PROOF,
             {'attributes': attributes, 'serviceEndpoint': service_endpoint}
         )
 
@@ -191,11 +229,36 @@ class ServiceFactory(object):
                 kwargs['attributes'],
                 kwargs['serviceEndpoint']
             )
+        elif service_type == ServiceTypes.NFT721_SALES:
+            return ServiceFactory.build_nft721_sales_service(
+                kwargs['attributes'],
+                kwargs['serviceEndpoint']
+            )
         elif service_type == ServiceTypes.NFT_ACCESS:
             return ServiceFactory.build_nft_access_service(
                 kwargs['attributes'],
                 kwargs['serviceEndpoint']
             )
+        elif service_type == ServiceTypes.NFT721_ACCESS:
+            return ServiceFactory.build_nft721_access_service(
+                kwargs['attributes'],
+                kwargs['serviceEndpoint']
+            )
+        elif service_type == ServiceTypes.NFT_ACCESS_PROOF:
+            return ServiceFactory.build_nft_access_proof_service(
+                kwargs['attributes'],
+                kwargs['serviceEndpoint']
+            )
+        elif service_type == ServiceTypes.NFT_ACCESS_SWAP:
+            return ServiceFactory.build_nft_access_swap_service(
+                kwargs['attributes'],
+                kwargs['serviceEndpoint']
+            )
+        elif service_type == ServiceTypes.NFT_SALES_WITH_ACCESS:
+            return ServiceFactory.build_nft_sales_with_access_service(
+                kwargs['attributes'],
+                kwargs['serviceEndpoint']
+            )
 
         raise ValueError(f'Unknown service type {service_type}')
 
@@ -253,6 +316,24 @@ class ServiceFactory(object):
                        values={'attributes': attributes},
                        index=ServiceTypesIndices.DEFAULT_ACCESS_PROOF_INDEX)
 
+    @staticmethod
+    def build_nft_access_proof_service(attributes, service_endpoint):
+        return Service(service_endpoint, ServiceTypes.NFT_ACCESS_PROOF,
+                       values={'attributes': attributes},
+                       index=ServiceTypesIndices.DEFAULT_NFT_ACCESS_PROOF_INDEX)
+
+    @staticmethod
+    def build_nft_access_swap_service(attributes, service_endpoint):
+        return Service(service_endpoint, ServiceTypes.NFT_ACCESS_SWAP,
+                       values={'attributes': attributes},
+                       index=ServiceTypesIndices.DEFAULT_NFT_ACCESS_SWAP_INDEX)
+
+    @staticmethod
+    def build_nft_sales_with_access_service(attributes, service_endpoint):
+        return Service(service_endpoint, ServiceTypes.NFT_SALES_WITH_ACCESS,
+                       values={'attributes': attributes},
+                       index=ServiceTypesIndices.DEFAULT_NFT_SALES_WITH_ACCESS_INDEX)
+
     @staticmethod
     def build_compute_service(attributes, service_endpoint):
         """
@@ -305,6 +386,34 @@ class ServiceFactory(object):
                        values={'attributes': attributes},
                        index=ServiceTypesIndices.DEFAULT_NFT_ACCESS_INDEX)
 
+
+    @staticmethod
+    def build_nft721_sales_service(attributes, service_endpoint):
+        """
+        Build a nft 721 sales service.
+
+        :param attributes: attributes of nft sales service, dict
+        :param service_endpoint: identifier of the service inside the asset DDO, str
+        :return: Service
+        """
+        return Service(service_endpoint, ServiceTypes.NFT721_SALES,
+                       values={'attributes': attributes},
+                       index=ServiceTypesIndices.DEFAULT_NFT721_SALES_INDEX)
+
+    @staticmethod
+    def build_nft721_access_service(attributes, service_endpoint):
+        """
+        Build a nft 721 access service.
+
+        :param attributes: attributes of nft sales service, dict
+        :param service_endpoint: identifier of the service inside the asset DDO, str
+        :return: Service
+        """
+        return Service(service_endpoint, ServiceTypes.NFT721_ACCESS,
+                       values={'attributes': attributes},
+                       index=ServiceTypesIndices.DEFAULT_NFT721_ACCESS_INDEX)
+
+
     @staticmethod
     def complete_access_service(did, service_endpoint, attributes, template_id,
                                 reward_contract_address=None, service_type=ServiceTypes.ASSET_ACCESS):
@@ -324,14 +433,17 @@ class ServiceFactory(object):
         }
 
         if reward_contract_address is not None:
-            param_map ['_rewardAddress'] = reward_contract_address
+            param_map['_rewardAddress'] = reward_contract_address
 
         try:
             param_map['_did'] = did_to_id(did)
             param_map['_amounts'] = attributes['main']['_amounts']
             param_map['_receivers'] = attributes['main']['_receivers']
             param_map['_tokenAddress'] = attributes['main']['_tokenAddress']
-            param_map['_numberNfts'] = attributes['main']['_numberNfts']
+            if service_type == ServiceTypes.NFT_ACCESS or \
+                service_type == ServiceTypes.NFT721_ACCESS or service_type == ServiceTypes.NFT_ACCESS_PROOF:
+                param_map['_numberNfts'] = attributes['main']['_numberNfts']
+                param_map['_contractAddress'] = attributes['main']['_contractAddress']
         except KeyError as e:
             logger.error(f'Error mapping field {e}')
 
@@ -379,6 +491,7 @@ class ServiceFactory(object):
         try:
             param_map['_amounts'] = attributes['main']['_amounts']
             param_map['_receivers'] = attributes['main']['_receivers']
+            # param_map['_returnAddress'] = attributes['main']['_returnAddress']
         except KeyError:
             pass
 
@@ -423,12 +536,17 @@ class ServiceFactory(object):
 
         if reward_contract_address is not None:
             param_map ['_rewardAddress'] = reward_contract_address
+        
+        # print(attributes['main'])
 
         try:
             param_map['_amounts'] = attributes['main']['_amounts']
             param_map['_receivers'] = attributes['main']['_receivers']
-            param_map['_numberNfts'] = attributes['main']['_numberNfts']
             param_map['_nftHolder'] = attributes['main']['_nftHolder']
+            param_map['_numberNfts'] = attributes['main']['_numberNfts']
+            param_map['_contractAddress'] = attributes['main']['_contractAddress']
+            param_map['_nftTransfer'] = attributes['main']['_nftTransfer']
+            param_map['_duration'] = attributes['main']['_duration']
         except KeyError:
             pass
 
