@@ -1,5 +1,5 @@
 class ServiceTypes:
-    """Types of Service allowed in ocean protocol DDO services."""
+    """Types of Service allowed in Nevermined DDO services."""
     AUTHORIZATION = 'authorization'
     METADATA = 'metadata'
     ASSET_ACCESS = 'access'
@@ -9,7 +9,10 @@ class ServiceTypes:
     NFT_SALES = 'nft-sales'
     NFT_ACCESS = 'nft-access'
     NFT721_ACCESS = 'nft721-access'
-
+    NFT721_SALES = 'nft721-sales'
+    NFT_ACCESS_PROOF = 'nft-access-proof'
+    NFT_ACCESS_SWAP = 'nft-access-swap'
+    NFT_SALES_WITH_ACCESS = 'nft-sales-with-access'
 
 class ServiceTypesIndices:
     DEFAULT_METADATA_INDEX = 0
@@ -22,7 +25,10 @@ class ServiceTypesIndices:
     DEFAULT_NFT_ACCESS_INDEX = 7
     DEFAULT_NFT721_ACCESS_INDEX = 9
     DEFAULT_ACCESS_PROOF_INDEX = 10
-
+    DEFAULT_NFT_ACCESS_PROOF_INDEX = 11
+    DEFAULT_NFT_ACCESS_SWAP_INDEX = 12
+    DEFAULT_NFT_SALES_WITH_ACCESS_INDEX = 13
+    DEFAULT_NFT721_SALES_INDEX = 14
 
 class ServiceAuthorizationTypes:
     SECRET_STORE = 'SecretStore'
